@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2004, 2006 Øyvind Kolås <pippin@gimp.org>
  */
@@ -20,36 +20,34 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
 #define THRESHOLD_SCRIPT \
-"level = user_value/2\n"\
-"for y=bound_y0, bound_y1 do\n"\
-"  for x=bound_x0, bound_x1 do\n"\
-"    v = get_value (x,y)\n"\
-"    v = v + get_value (1, x,y)\n"\
-"    if v>level then\n"\
-"      v=1.0\n"\
-"    else\n"\
-"      v=0.0\n"\
-"    end\n"\
-"    set_value (x,y,v)\n"\
-"  end\n"\
-"  progress (y/height)\n"\
-"end"
+"r,g,b = get_rgb (x,y)\n"\
+"set_rgb (x,y,g,g,g)\n"\
 
-gegl_chant_multiline (script, _("Script"), THRESHOLD_SCRIPT,
-         _("The lua script containing the implementation of this operation."))
-gegl_chant_file_path (file, _("File"), "", _("a stored lua script on disk implementing an operation."))
-gegl_chant_double (user_value, _("User value"), -1000.0, 1000.0, 1.0,
-         _("(appears in the global variable 'user_value' in lua."))
+property_string (script, _("Script"), THRESHOLD_SCRIPT)
+    description(_("The lua script containing the implementation of this operation."))
+    ui_meta    ("multiline", "true")
+
+property_string (error, _("Lua error"), "")
+    description(_("Parse/compile error, if any."))
+    ui_meta    ("error", "true")
+
+property_file_path (file, _("File"), "")
+    description(_("A stored lua script on disk implementing an operation."))
+
+property_double (user_value, _("User value"), 1.0)
+    description(_("Available as a global variable 'user_value' in lua."))
+    value_range (-1000.0, 1000.0)
 
 #else
 
-#define GEGL_CHANT_TYPE_COMPOSER
-#define GEGL_CHANT_C_FILE       "gluas.c"
+#define GEGL_OP_COMPOSER
+#define GEGL_OP_NAME     gluas
+#define GEGL_OP_C_SOURCE gluas.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
@@ -60,6 +58,9 @@ typedef struct Priv
   GeglBuffer *in_drawable;
   GeglBuffer *aux_drawable;
   GeglBuffer *out_drawable;
+
+  GeglSampler *in_sampler;
+  GeglSampler *aux_sampler;
   const Babl *rgba_float;
 
   gint        bx1, by1;
@@ -97,7 +98,7 @@ static int l_progress  (lua_State * lua);
 static int l_flush     (lua_State * lua);
 static int l_print     (lua_State * lua);
 
-static const luaL_reg gluas_functions[] =
+static const luaL_Reg gluas_functions[] =
 {
     {"set_rgba",    l_set_rgba},
     {"get_rgba",    l_get_rgba},
@@ -122,7 +123,7 @@ static const luaL_reg gluas_functions[] =
 };
 static void
 register_functions (lua_State      *L,
-                    const luaL_reg *l)
+                    const luaL_Reg *l)
 {
   for (;l->name; l++)
     lua_register (L, l->name, l->func);
@@ -146,7 +147,7 @@ drawable_lua_process (GeglOperation       *op,
     lua_State *L;
     Priv p;
 
-    L = lua_open ();
+    L = luaL_newstate ();
     luaL_openlibs (L);
 
     register_functions (L, gluas_functions);
@@ -172,9 +173,17 @@ drawable_lua_process (GeglOperation       *op,
     lua_pushlightuserdata (L, &p);
     lua_settable (L, LUA_REGISTRYINDEX);
 
-    p.in_drawable = drawable;
+    p.in_drawable  = drawable;
     p.aux_drawable = aux;
     p.out_drawable = result;
+    if (drawable)
+      p.in_sampler = gegl_buffer_sampler_new (drawable, p.rgba_float, GEGL_SAMPLER_NEAREST);
+    else
+      p.in_sampler = NULL;
+    if (aux)
+      p.aux_sampler = gegl_buffer_sampler_new (aux, p.rgba_float, GEGL_SAMPLER_NEAREST);
+    else
+      p.aux_sampler = NULL;
 
     lua_pushnumber (L, (double) p.bx1);
     lua_setglobal (L, "bound_x0");
@@ -190,17 +199,34 @@ drawable_lua_process (GeglOperation       *op,
 
       luaL_loadstring (L, "os.setlocale ('C', 'numeric')");
 
+
+      /* insert default loop start/end filling the selection */
       if (file && file[0]!='\0')
         status = luaL_loadfile (L, file);
       else if (buffer)
-        status = luaL_loadbuffer (L, buffer, strlen (buffer), "buffer");
+      {
+        GString *str = g_string_new (buffer);
+
+        if (!strstr (buffer, "for x"))
+        {
+          g_string_prepend (str, "for y=bound_y0, bound_y1 do\n for x=bound_x0, bound_x1 do\n");
+          g_string_append (str, " end \n progress (y/height)\n end\n");
+        }
+        status = luaL_loadbuffer (L, str->str, str->len, "buffer");
+
+        g_string_free (str, TRUE);
+      }
 
       if (status == 0)
         status = lua_pcall (L, 0, LUA_MULTRET, 0);
 
       if (status != 0)
-        g_warning ("lua error: %s", lua_tostring (L, -1));
+        gegl_node_set (op->node, "error", lua_tostring (L, -1), NULL);
     }
+  if (p.in_sampler)
+    g_object_unref (p.in_sampler);
+  if (p.aux_sampler)
+    g_object_unref (p.aux_sampler);
 }
 
 #if 0
@@ -242,9 +268,8 @@ get_rgba_pixel (void       *data,
       gint i;
       if (!p->in_drawable)
         return;
-      gegl_buffer_sample (p->in_drawable, x, y, NULL, buf,
-                          p->rgba_float,
-                          GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
+      gegl_sampler_get (p->in_sampler, x, y, NULL, buf,
+                        GEGL_ABYSS_NONE);
       for (i = 0; i < 4; i++)
         pixel[i] = buf[i];
     }
@@ -253,9 +278,8 @@ get_rgba_pixel (void       *data,
       gint i;
       if (!p->aux_drawable)
         return;
-      gegl_buffer_sample (p->aux_drawable, x, y, NULL, buf,
-                          p->rgba_float,
-                          GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
+      gegl_sampler_get (p->aux_sampler, x, y, NULL, buf,
+                        GEGL_ABYSS_NONE);
       for (i = 0; i < 4; i++)
         pixel[i] = buf[i];
     }
@@ -321,11 +345,11 @@ l_in_height(lua_State * lua)
 static int
 l_flush (lua_State * lua)
 {
-  Priv *p;
+  //Priv *p;
 
   lua_pushstring(lua, "priv");
   lua_gettable(lua, LUA_REGISTRYINDEX);
-  p = lua_touserdata(lua, -1);
+  //p = lua_touserdata(lua, -1);
 
   return 0;
 }
@@ -333,13 +357,13 @@ l_flush (lua_State * lua)
 static int
 l_progress(lua_State * lua)
 {
-  lua_Number percent;
 
   if (!lua_gettop(lua))
     return 0;
+#if 0
+  lua_Number percent;
   percent = lua_tonumber(lua, -1);
 
-#if 0
   g_warning ("progress,.. %f", percent);
   gimp_progress_update((double) percent);
 #endif
@@ -914,8 +938,8 @@ static int l_get_hsv (lua_State * lua)
 static void
 prepare (GeglOperation *operation)
 {
-  gegl_operation_set_format (operation, "input", babl_format ("RGBA float"));
-  gegl_operation_set_format (operation, "aux", babl_format ("RGBA float"));
+  gegl_operation_set_format (operation, "input",  babl_format ("RGBA float"));
+  gegl_operation_set_format (operation, "aux",    babl_format ("RGBA float"));
   gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
 }
 
@@ -940,7 +964,7 @@ process (GeglOperation       *operation,
          const GeglRectangle *result,
          gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   if (o->file && g_file_test (o->file, G_FILE_TEST_IS_REGULAR))
     {
@@ -964,7 +988,7 @@ get_required_for_output (GeglOperation        *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass         *operation_class;
   GeglOperationComposerClass *composer_class;
@@ -974,15 +998,16 @@ gegl_chant_class_init (GeglChantClass *klass)
 
   composer_class->process = process;
   operation_class->prepare = prepare;
+  operation_class->threaded = FALSE;
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->get_required_for_output = get_required_for_output;
 
   gegl_operation_class_set_keys (operation_class,
   "name"        , "gegl:gluas",
   "categories"  , "script:misc",
+  "title"       , _("gluas"),
   "description" ,
-        _("A general purpose filter/composer implementation proxy for the"
-          " lua programming language."),
+        _("Pixel level filter/composer scripting prototyping with lua. See http://pippin.gimp.org/image-processing/ for further details."),
         NULL);
 }
 
